-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

local runeColors = {
	[RUNETYPE_BLOOD] = {1, 0, 0},
	[RUNETYPE_UNHOLY] = {0, 0.5, 0},
	[RUNETYPE_FROST] = {0, 1, 1},
	[RUNETYPE_DEATH] = {0.8, 0.1, 1},
}
local runeMapping = {
	[1] = "BLOOD",
	[2] = "UNHOLY",
	[3] = "FROST",
	[4] = "DEATH",
}

lib.AddRunesFrame = function(pos)
	pos = pos or {"TOP",Heddframe,"BOTTOM",0,-20}
	Heddframe.runes = Heddframe.runes or CreateFrame('Frame',"$parent_runes",Heddframe)
	Heddframe.runes:SetHeight(cfg.rsize*2)
	Heddframe.runes:SetWidth(cfg.rsize*3+cfg.rspace*2)
	Heddframe.runes:SetPoint(unpack(pos))
	Heddframe.runes:Show()
	Heddframe.runes.lu=0
	hedlib.CreateBD(Heddframe.runes,20,{1,0,0,1})
	Heddframe.runes.bd:Hide()
	for i = 1, 6 do
		Heddframe.runes[i] = Heddframe.runes[i] or CreateFrame('StatusBar', "$parent_"..i, Heddframe.runes)
		Heddframe.runes[i].rune=i
		Heddframe.runes[i]:SetStatusBarTexture(cfg.statusbar_texture)
		Heddframe.runes[i]:SetHeight(cfg.rsize);
		Heddframe.runes[i]:SetWidth(cfg.rsize);
		hedlib.CreateBG(Heddframe.runes[i],0)
		Heddframe.runes[i].bg:SetTexture(cfg.statusbar_texture)
		Heddframe.runes[i].bg:SetAlpha(0.3)
--[[		if cfg.omnicc then
			Heddframe.runes[i].cooldown = _G["Hrune"..i.."Cooldown"] or CreateFrame("Cooldown", "$parentCooldown", Heddframe.runes[i], "CooldownFrameTemplate")
			Heddframe.runes[i].cooldown:SetAllPoints(Heddframe.runes[i])
--			Heddframe.runes[i].cooldown:SetReverse()
--			Heddframe.runes[i].cooldown:SetAlpha(0)
		else]]
			Heddframe.runes[i].text=Heddframe.runes[i].text or hedlib.CreateFont(Heddframe.runes[i], cfg.textfont, nil, "OUTLINE")
			Heddframe.runes[i].text:SetAllPoints(Heddframe.runes[i])
			Heddframe.runes[i].text:SetTextColor(1,1,0)
			--Heddframe.runes[i].text:SetText(i)
--		end
		Heddframe.runes[i].lu=GetTime()
		Heddframe.runes[i].freq=cfg.freq
		Heddframe.runes[i]:SetID(i)
		Heddframe.runes[i].tooltipText=nil
		Heddframe.runes[i]:SetMinMaxValues(0,10)
		Heddframe.runes[i]:SetValue(10)
		if cfg.rshine then
			Heddframe.runes[i].shine = Heddframe.runes[i]:CreateTexture(nil,"OVERLAY")
			Heddframe.runes[i].shine:SetHeight(cfg.rsize*3)
			Heddframe.runes[i].shine:SetWidth(cfg.rsize*3)
			Heddframe.runes[i].shine:SetPoint("CENTER",Heddframe.runes[i],"CENTER")
			Heddframe.runes[i].shine:SetTexture("Interface\\ComboFrame\\ComboPoint")
			Heddframe.runes[i].shine:SetTexCoord(0.5625,1,0,1)
			Heddframe.runes[i].shine:SetAlpha(0)
		end
--		Heddframe.runes[i].start=GetTime()
--		Heddframe.runes[i].duration=0
--		Heddframe.runes[i].runeReady=true
		--lib.updateRune(Heddframe.runes,i)
	end

	Heddframe.runes[1]:SetPoint("TOPLEFT", Heddframe.runes, "TOPLEFT", 0, 0);
	Heddframe.runes[2]:SetPoint("BOTTOMLEFT", Heddframe.runes, "BOTTOMLEFT", 0, 0);
	Heddframe.runes[3]:SetPoint("LEFT", Heddframe.runes[1], "RIGHT", cfg.rspace, 0);
	Heddframe.runes[4]:SetPoint("LEFT", Heddframe.runes[2], "RIGHT", cfg.rspace, 0);
	Heddframe.runes[5]:SetPoint("LEFT", Heddframe.runes[3], "RIGHT", cfg.rspace, 0);
	Heddframe.runes[6]:SetPoint("LEFT", Heddframe.runes[4], "RIGHT", cfg.rspace, 0);
	for i_runes = 1, 6 do
		lib.InitRune(i_runes)
	end
	lib.Hedd_slash("runes "..HeddDB.runes)
	cfg.Update=true
end



lib.AddRuneSpell = function(spell,ids,runemask,addbuff)
	if lib.AddSpell(spell,ids,addbuff) then
		--cfg.spells[spell].rune=true
		cfg.spells[spell].powerType="rune"
		cfg.spells[spell].nousecheck=true
		cfg.spells[spell].red = runemask[1]
		cfg.spells[spell].green = runemask[2]
		cfg.spells[spell].blue = runemask[3]
		cfg.spells[spell].numrunes = cfg.spells[spell].red + cfg.spells[spell].green + cfg.spells[spell].blue
		lib.UpdateSpell(spell)
		return true
	end
	return false
end

local rune_old={}
local runeType
lib.InitRune = function(rune)
	if not rune then return nil end
	runeType=GetRuneType(rune)
	if runeType then
		cfg.Runes[rune].runeType=runeType
		cfg.Runes[rune].start, cfg.Runes[rune].duration, cfg.Runes[rune].runeReady = GetRuneCooldown(rune)
		cfg.RuneTypesInfo[runeType].num=cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num+1
		cfg.RuneTypesInfo[runeType].runes[rune]=true
		if cfg.Runes[rune].runeReady then
			cfg.RuneTypesInfo[runeType].ready=cfg.RuneTypesInfo[runeType].ready+1
		end
		lib.UpdateRuneFrame(rune)
	end
end

local runeType
lib.UpdateRune = function(rune)
	if rune and rune>=1 and rune<=6 then
		runeType=GetRuneType(rune)
		if not runeType then return false end
		hedlib.shallowCopy(cfg.Runes[rune],rune_old)
		cfg.Runes[rune].runeType=runeType
		if not cfg.Runes[rune].runeType then print("NO RUNE TYPE") end
		cfg.Runes[rune].start, cfg.Runes[rune].duration, cfg.Runes[rune].runeReady = GetRuneCooldown(rune)
		if not hedlib.ArrayNotChanged(cfg.Runes[rune],rune_old) then
			if rune_old.runeType~=cfg.Runes[rune].runeType then
				cfg.RuneTypesInfo[rune_old.runeType].num=cfg.RuneTypesInfo[rune_old.runeType].num-1
				if cfg.RuneTypesInfo[rune_old.runeType].num<0 then
					cfg.RuneTypesInfo[rune_old.runeType].num=0
				end
				cfg.RuneTypesInfo[rune_old.runeType].runes[rune]=nil
				
				cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num=cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num+1
				cfg.RuneTypesInfo[cfg.Runes[rune].runeType].runes[rune]=true
			end
			if cfg.Runes[rune].runeReady~=rune_old.runeReady then
				if cfg.Runes[rune].runeReady then
					cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready=cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready+1
				else
					cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready=cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready-1
					if cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready<0 then 
						cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready=0
					end
				end
			end
			if cfg.RuneTypesInfo[rune_old.runeType].ready>cfg.RuneTypesInfo[rune_old.runeType].num then
				cfg.RuneTypesInfo[rune_old.runeType].ready=cfg.RuneTypesInfo[rune_old.runeType].num
			end
			if cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready>cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num then
				cfg.RuneTypesInfo[cfg.Runes[rune].runeType].ready=cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num
			end
			lib.UpdateRuneFrame(rune)
			cfg.Update=true
			return true
		end
	end
	return false
end
lib.PrintRune=function(rune)
	print ("Rune="..rune.." - "..cfg.Runes[rune].runeType.."("..cfg.RuneTypesInfo[cfg.Runes[rune].runeType].num..") ".." start="..cfg.Runes[rune].start.." duration="..cfg.Runes[rune].duration.." runeReady="..tostring(cfg.Runes[rune].runeReady))
end

lib.PrintType=function(runeType)
	print ("Type="..runeMapping[runeType].." "..cfg.RuneTypesInfo[runeType].ready)
end


lib.UpdateRuneFrame = function(rune)
	Heddframe.runes[rune]:SetStatusBarColor(unpack(runeColors[cfg.Runes[rune].runeType]))
	Heddframe.runes[rune].bg:SetVertexColor(unpack(runeColors[cfg.Runes[rune].runeType]))
	--Heddframe.runes[rune]:Show()
	Heddframe.runes[rune]:SetMinMaxValues(0,cfg.Runes[rune].duration*10)

	if not cfg.Runes[rune].runeReady then
		Heddframe.runes[rune]:SetScript("OnUpdate", lib.OnUpdateRune)
	end
	lib.OnUpdateRune(Heddframe.runes[rune])
end

lib.OnUpdateRune = function (self, elapsed) --Heddframe.runes[rune]=self
	if cfg.Runes[self.rune].start==0 then
		self.cd=0
		self:SetValue(cfg.Runes[self.rune].duration*10)
		self.text:SetText("")
		self:SetScript("OnUpdate", nil)
	else
		self.cd=hedlib.round(lib.GetRuneCD(self.rune))
		self:SetValue((GetTime() - cfg.Runes[self.rune].start)*10)
		if self.cd<=cfg.Runes[self.rune].duration and self.cd>0 then
			self.text:SetText(self.cd)
		else
			self.text:SetText("")
		end
	end	
end

local i_runes
lib.UpdateAllRunes = function()
	allrunesupdate=nil
	for i_runes = 1, 6 do
		if lib.UpdateRune(i_runes) then allrunesupdate=true end
	end
	return allrunesupdate
end

local runetypecd=9999
local runetypecd_current=0
lib.RuneTypeCD = function(runeType,longest)
	runetypecd=nil
	runetypecd_current=0
	if cfg.RuneTypesInfo[runeType].num==0 then
		return 9999
	end
	for rune,value in pairs(cfg.RuneTypesInfo[runeType].runes) do
		runetypecd_current=lib.GetRuneCD(rune)
		runetypecd=runetypecd or runetypecd_current
		if longest then
			if runetypecd_current>runetypecd then runetypecd=runetypecd_current end
		else
			if runetypecd_current<runetypecd then runetypecd=runetypecd_current end
		end
	end
	return runetypecd
end

lib.GetRuneCD = function(rune)
	if not cfg.Runes or not cfg.Runes[rune] then return 0 end
	return math.max(0,cfg.Runes[rune].duration - (GetTime() - cfg.Runes[rune].start))
end

lib.FullDeplatedRunes = function()
	allrunes=0
	for i_runes = 1,3 do
		numrunes=0
		c_rune = 0
		tl_rune=99999
		start,finish=i_runes*2-1,i_runes*2
		
		for i_runes = start, finish do
			tl_rune_any =  cfg.Runes[i_runes].duration - (GetTime() - cfg.Runes[i_runes].start)
			tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
			if tl_rune_any==0 then
				numrunes=numrunes+1
			end
		end
		allrunes = (numrunes==0) and (allrunes+1) or 0
	end
	return allrunes
end

lib.getruneCD = function(runeType,useDeath)
	c_rune = 0
	tl_rune=99999
	ReadyRuneFound=nil
	
	if runeType==1 then
		start,finish=1,2
	elseif runeType==2 then
		start,finish=3,4
	else
		start,finish=5,6
	end
	
	for i_runes = start, finish do
		if ReadyRuneFound then return 0 end
		if not ExcludeRunes[i_runes] and cfg.Runes[i_runes].runeType==runeType then
			tl_rune_any =  cfg.Runes[i_runes].duration - (GetTime() - cfg.Runes[i_runes].start)
			tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
			if tl_rune_any==0 then
				ReadyRuneFound=true
				ExcludeRunes[i_runes]=true
			else
				if tl_rune_any<tl_rune then
					tl_rune=tl_rune_any
					c_rune=i_runes
				end
			end
		end
	end
	if ReadyRuneFound then return 0 end
	
	if useDeath then
		for i_runes = 1, 6 do
			if ReadyRuneFound then return 0 end
			if not ExcludeRunes[i_runes] and cfg.Runes[i_runes].runeType==4 then
				tl_rune_any =  cfg.Runes[i_runes].duration - (GetTime() - cfg.Runes[i_runes].start)
				tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
				if tl_rune_any==0 then
					ReadyRuneFound=true
					ExcludeRunes[i_runes]=true
				else
					if tl_rune_any<tl_rune then
						tl_rune=tl_rune_any
						c_rune=i_runes
					end
				end
			end
		end	
	end
	if ReadyRuneFound then return 0 end
	ExcludeRunes[c_rune]=true
	
	return tl_rune
end

lib.RuneCDCheck = function (spell,useDeath,twospells,useGCD,minWait)
	minWait=minWait or 0
	ExcludeRunes = {}

	if cfg.spells[spell] then
		if useGCD and cfg.gcd then
			tl_spell=cfg.spells[cfg.gcd].cd - (GetTime() - cfg.spells[cfg.gcd].start)
		else
			tl_spell=cfg.spells[spell].cd - (GetTime() - cfg.spells[spell].start)
		end
		tl_spell = (tl_spell>=0) and tl_spell or 0
		
		tl_rune_r,tl_rune_g,tl_rune_b = 0,0,0
		n_rune_r,n_rune_g,n_rune_b = 0,0,0
		
		for j = 1,2 do
			if cfg.spells[spell].red==1 then
				tl_rune_r=lib.getruneCD(1,useDeath)
				if ReadyRuneFound then n_rune_r = n_rune_r+1 end
			else
				n_rune_r=2
			end
		
			if cfg.spells[spell].green==1 then
				tl_rune_g=lib.getruneCD(2,useDeath)
				if ReadyRuneFound then n_rune_g = n_rune_g+1 end
			else
				n_rune_g=2
			end
		
			if cfg.spells[spell].blue==1 then
				tl_rune_b=lib.getruneCD(3,useDeath)
				if ReadyRuneFound then n_rune_b = n_rune_b+1 end
			else
				n_rune_b=2
			end
			
			if not twospells then break end
		end
		
		if twospells and n_rune_r==2 and n_rune_g==2 and n_rune_b==2 then
			lib.UpdateBar(spell,0)
			return true
		end
		
		if n_rune_r>0 and n_rune_g>0 and n_rune_b>0 then
			tl_spell=tl_spell
		else
			if tl_spell<tl_rune_r then tl_spell=tl_rune_r end
			if tl_spell<tl_rune_g then tl_spell=tl_rune_g end
			if tl_spell<tl_rune_b then tl_spell=tl_rune_b end
		end
		
		if tl_spell==0 or tl_spell<=minWait or tl_spell==cfg.timeleft then
			lib.UpdateBar(spell,tl_spell)
			cfg.ctimeleft=tl_spell
			return true
		else
			if cfg.ctimeleft>tl_spell then
				cfg.ctimeleft=tl_spell
			end
		end
	end
	return nil
end

lib.GetRuneSpellCD=function(spell,useDeath,useGCD)
	ExcludeRunes = {}
	if cfg.spells[spell] then
		if useGCD then
			tl_spell=cfg.spells[cfg.gcd].cd - (GetTime() - cfg.spells[cfg.gcd].start)
		else
			tl_spell=cfg.spells[spell].cd - (GetTime() - cfg.spells[spell].start)
		end
		tl_spell = (tl_spell>=0) and tl_spell or 0
		
		tl_rune_r,tl_rune_g,tl_rune_b = 0,0,0
		n_rune_r,n_rune_g,n_rune_b = 0,0,0

		if cfg.spells[spell].red==1 then
			tl_rune_r=lib.getruneCD(1,useDeath)
			if ReadyRuneFound then n_rune_r = n_rune_r+1 end
		else
			n_rune_r=1
		end
		
		if cfg.spells[spell].green==1 then
			tl_rune_g=lib.getruneCD(2,useDeath)
			if ReadyRuneFound then n_rune_g = n_rune_g+1 end
		else
			n_rune_g=1
		end
		
		if cfg.spells[spell].blue==1 then
			tl_rune_b=lib.getruneCD(3,useDeath)
			if ReadyRuneFound then n_rune_b = n_rune_b+1 end
		else
			n_rune_b=1
		end
			
		if n_rune_r>0 and n_rune_g>0 and n_rune_b>0 then
			tl_spell=tl_spell
		else
			if tl_spell<tl_rune_r then tl_spell=tl_rune_r end
			if tl_spell<tl_rune_g then tl_spell=tl_rune_g end
			if tl_spell<tl_rune_b then tl_spell=tl_rune_b end
		end

		return tl_spell
	end
	return 0
end

lib.GetDepletedRune = function(runeType)
	return (cfg.RuneTypesInfo[runeType].num-cfg.RuneTypesInfo[runeType].ready)
end

lib.GetDepletedSet = function(set)
	if not cfg.RuneSet[set] then return 0 end
	return ((cfg.Runes[cfg.RuneSet[set][1]].runeReady and 0 or 1)+(cfg.Runes[cfg.RuneSet[set][2]].runeReady and 0 or 1))
end

lib.GetDepletedRunes = function()
	return ((cfg.Runes[1].runeReady and 0 or 1)+
	(cfg.Runes[2].runeReady and 0 or 1)+
	(cfg.Runes[3].runeReady and 0 or 1)+
	(cfg.Runes[4].runeReady and 0 or 1)+
	(cfg.Runes[5].runeReady and 0 or 1)+
	(cfg.Runes[6].runeReady and 0 or 1))
end


lib.GetNumRune = function(runeType)
	return cfg.RuneTypesInfo[runeType].num
end

lib.RunesOptionsToggle = function()
	--if cfg.class~="DEATHKNIGHT" then return end
	if HeddDB.runes=="always" then
		lib.ShowFrame(Heddframe.runes)
	elseif HeddDB.runes=="incombat" then
		if cfg.combat then
			lib.ShowFrame(Heddframe.runes)
		else
			lib.HideFrame(Heddframe.runes)
		end
	elseif HeddDB.runes=="hide" then
		lib.HideFrame(Heddframe.runes)
	end
end

lib.DK_events = function()
	cfg.RuneTypesInfo={}
	for i_runes = 1, 4 do
		cfg.RuneTypesInfo[i_runes]={num=0,ready=0,runes={}}
	end
	cfg.Runes={}
	for i_runes = 1, 6 do
		cfg.Runes[i_runes]={}
--		lib.InitRune(i_runes)
	end
	cfg.RuneSet={[1]={1,2},[2]={3,4},[3]={5,6}}

	lib.AddRunesFrame()
	
	function Heddclassevents.PLAYER_UNGHOST()
		if lib.UpdateAllRunes() then
			lib.UpdateAllSpells("rune")
			cfg.Update=true
		end
	end
	
	function Heddclassevents.RUNE_POWER_UPDATE(rune,isEnergize)
		--if rune then print(rune.." "..(isEnergize and "true" or "false")) end
		--if isEnergize then print(rune.." energized") end
		if lib.UpdateRune(rune) then
			lib.UpdateAllSpells("rune")
			cfg.Update=true
		end
	end
	
	Heddclassevents.RUNE_TYPE_UPDATE=Heddclassevents.RUNE_POWER_UPDATE
end